import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { ProductModel, CartItem } from '../interface/product.interface';

/**
 * CartExtService - Extended cart with discount
 */
@Injectable()
export class CartExtService {
  private cartItemsSubject = new BehaviorSubject<CartItem[]>([]);
  private cartCountSubject = new BehaviorSubject<number>(0);
  private discountPercent = 10; // 10% discount

  cartItems$: Observable<CartItem[]> = this.cartItemsSubject.asObservable();
  cartCount$: Observable<number> = this.cartCountSubject.asObservable();

  constructor() {
    console.log('🌟 CartExtService created');
    this.loadCartFromStorage();
  }

  addToCart(product: ProductModel, quantity: number = 1): void {
    const currentItems = this.cartItemsSubject.value;
    const existingItemIndex = currentItems.findIndex(item => item.product.id === product.id);

    let updatedItems: CartItem[];
    
    if (existingItemIndex >= 0) {
      updatedItems = currentItems.map((item, index) => 
        index === existingItemIndex 
          ? { ...item, quantity: item.quantity + quantity }
          : item
      );
    } else {
      updatedItems = [...currentItems, { product, quantity }];
    }

    this.updateCart(updatedItems);
    console.log(`🚀 [EXT] Added ${product.name} to cart with external processing`);
    
    // Simulate external API call
    this.syncWithExternalAPI('ADD', product.id, quantity);
  }

  removeFromCart(productId: number): void {
    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.filter(item => item.product.id !== productId);
    this.updateCart(updatedItems);
    console.log(`🔥 [EXT] Removed product ${productId} with external sync`);
    
    this.syncWithExternalAPI('REMOVE', productId, 0);
  }

  updateQuantity(productId: number, quantity: number): void {
    if (quantity <= 0) {
      this.removeFromCart(productId);
      return;
    }

    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.map(item => 
      item.product.id === productId 
        ? { ...item, quantity }
        : item
    );

    this.updateCart(updatedItems);
    console.log(`⚡ [EXT] Updated quantity for product ${productId} = ${quantity}`);
    
    this.syncWithExternalAPI('UPDATE', productId, quantity);
  }

  /**
   * Calculate total with discount (Extended feature)
   */
  calculateTotal(): number {
    const subtotal = this.cartItemsSubject.value.reduce(
      (total, item) => total + (item.product.price * item.quantity),
      0
    );
    
    // Apply discount if total > 1000
    if (subtotal > 1000) {
      const discount = subtotal * (this.discountPercent / 100);
      return subtotal - discount;
    }
    
    return subtotal;
  }

  /**
   * Get discount amount (Extended feature)
   */
  getDiscountAmount(): number {
    const subtotal = this.getSubtotal();
    if (subtotal > 1000) {
      return subtotal * (this.discountPercent / 100);
    }
    return 0;
  }

  /**
   * Get subtotal before discount
   */
  getSubtotal(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + (item.product.price * item.quantity),
      0
    );
  }

  getTotalItems(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + item.quantity,
      0
    );
  }

  clearCart(): void {
    this.updateCart([]);
    console.log('💫 [EXT] Cleared cart with external notification');
    
    this.syncWithExternalAPI('CLEAR', 0, 0);
  }

  getCurrentItems(): CartItem[] {
    return this.cartItemsSubject.value;
  }

  /**
   * Simulate external API sync (Extended feature)
   */
  private syncWithExternalAPI(action: string, productId: number, quantity: number): void {
    // Simulate API call
    setTimeout(() => {
      console.log(`🌐 [EXT] Synced with external API: ${action} - Product ${productId}, Qty: ${quantity}`);
    }, Math.random() * 1000);
  }

  private updateCart(items: CartItem[]): void {
    this.cartItemsSubject.next(items);
    this.cartCountSubject.next(this.getTotalItems());
    this.saveCartToStorage(items);
  }

  private saveCartToStorage(items: CartItem[]): void {
    try {
      localStorage.setItem('cart_ext', JSON.stringify(items));
    } catch (error) {
      console.error('❌ [EXT] Cannot save cart to localStorage:', error);
    }
  }

  private loadCartFromStorage(): void {
    try {
      const savedCart = localStorage.getItem('cart_ext');
      if (savedCart) {
        const items: CartItem[] = JSON.parse(savedCart);
        this.cartItemsSubject.next(items);
        this.cartCountSubject.next(this.getTotalItems());
        console.log('📥 [EXT] Loaded cart from localStorage');
      }
    } catch (error) {
      console.error('❌ [EXT] Cannot load cart from localStorage:', error);
    }
  }
}
