import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { ProductModel, CartItem } from '../interface/product.interface';

/**
 * CartService - Demo Dependency Injection
 */
@Injectable({
  providedIn: 'root'
})
export class CartService {
  private cartItemsSubject = new BehaviorSubject<CartItem[]>([]);
  private cartCountSubject = new BehaviorSubject<number>(0);

  cartItems$: Observable<CartItem[]> = this.cartItemsSubject.asObservable();
  cartCount$: Observable<number> = this.cartCountSubject.asObservable();

  constructor() {
    console.log('🛒 CartService created');
    this.loadCartFromStorage();
  }

  /**
   * Add product to cart
   */
  addToCart(product: ProductModel, quantity: number = 1): void {
    const currentItems = this.cartItemsSubject.value;
    const existingItemIndex = currentItems.findIndex(item => item.product.id === product.id);

    let updatedItems: CartItem[];
    
    if (existingItemIndex >= 0) {
      updatedItems = currentItems.map((item, index) => 
        index === existingItemIndex 
          ? { ...item, quantity: item.quantity + quantity }
          : item
      );
    } else {
      updatedItems = [...currentItems, { product, quantity }];
    }

    this.updateCart(updatedItems);
    console.log(`✅ Added ${product.name} to cart`);
  }

  /**
   * Remove product from cart
   */
  removeFromCart(productId: number): void {
    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.filter(item => item.product.id !== productId);
    this.updateCart(updatedItems);
    console.log(`🗑️ Removed product ID ${productId}`);
  }

  /**
   * Update quantity
   */
  updateQuantity(productId: number, quantity: number): void {
    if (quantity <= 0) {
      this.removeFromCart(productId);
      return;
    }

    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.map(item => 
      item.product.id === productId 
        ? { ...item, quantity }
        : item
    );

    this.updateCart(updatedItems);
    console.log(`📝 Updated quantity for product ID ${productId} = ${quantity}`);
  }

  /**
   * Calculate total
   */
  calculateTotal(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + (item.product.price * item.quantity),
      0
    );
  }

  /**
   * Get total items count
   */
  getTotalItems(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + item.quantity,
      0
    );
  }

  /**
   * Clear cart
   */
  clearCart(): void {
    this.updateCart([]);
    console.log('🧹 Cart cleared');
  }

  /**
   * Get current items
   */
  getCurrentItems(): CartItem[] {
    return this.cartItemsSubject.value;
  }

  /**
   * Update cart and sync with localStorage
   */
  private updateCart(items: CartItem[]): void {
    this.cartItemsSubject.next(items);
    this.cartCountSubject.next(this.getTotalItems());
    this.saveCartToStorage(items);
  }

  /**
   * Save to localStorage
   */
  private saveCartToStorage(items: CartItem[]): void {
    try {
      localStorage.setItem('cart', JSON.stringify(items));
    } catch (error) {
      console.error('❌ Cannot save to localStorage:', error);
    }
  }

  /**
   * Load from localStorage
   */
  private loadCartFromStorage(): void {
    try {
      const savedCart = localStorage.getItem('cart');
      if (savedCart) {
        const items: CartItem[] = JSON.parse(savedCart);
        this.cartItemsSubject.next(items);
        this.cartCountSubject.next(this.getTotalItems());
        console.log('📦 Loaded cart from localStorage');
      }
    } catch (error) {
      console.error('❌ Cannot load from localStorage:', error);
    }
  }
}
