import { Component, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common';
import { AuthorDetailComponent } from '../author-detail/author-detail.component';
import { Author } from '../../data-access/interface/author.interface';
import { authors } from '../../data-access/const/authors.const';

@Component({
  selector: 'app-author-list',
  standalone: true,
  imports: [CommonModule, AuthorDetailComponent],
  template: `
    <div class="tw-max-w-4xl tw-mx-auto tw-p-6">
      <div class="tw-flex tw-flex-col sm:tw-flex-row tw-justify-between tw-items-start sm:tw-items-center tw-mb-6 tw-gap-4">
        <h2 class="tw-text-2xl tw-font-bold tw-text-gray-800">Authors List ({{ authors.length }} authors)</h2>
        <button 
          (click)="resetAuthors()"
          class="tw-bg-white hover:tw-bg-gray-100 tw-text-black tw-border-2 tw-border-black tw-px-6 tw-py-2 tw-rounded-lg tw-transition-colors tw-font-bold tw-shadow-md hover:tw-shadow-lg"
        >
          🔄 Reset List
        </button>
      </div>
      
      <!-- Event Log -->
      <div *ngIf="eventLog.length > 0" class="tw-mb-6 tw-bg-gray-100 tw-p-4 tw-rounded-lg tw-border">
        <h3 class="tw-font-semibold tw-text-gray-700 tw-mb-2">🔔 Event Log:</h3>
        <div class="tw-space-y-1 tw-max-h-40 tw-overflow-y-auto">
          <div *ngFor="let event of eventLog; let i = index" 
               class="tw-text-sm tw-text-gray-600 tw-flex tw-items-start tw-space-x-2">
            <span class="tw-font-mono tw-text-xs tw-bg-gray-200 tw-px-2 tw-py-1 tw-rounded tw-flex-shrink-0">{{ i + 1 }}</span>
            <span class="tw-flex-1">{{ event }}</span>
          </div>
        </div>
        <div class="tw-flex tw-justify-between tw-items-center tw-mt-3">
          <span class="tw-text-xs tw-text-gray-500">Total events: {{ eventLog.length }}</span>
          <button 
            (click)="clearEventLog()"
            class="tw-text-xs tw-bg-red-100 tw-text-red-600 hover:tw-bg-red-200 tw-px-3 tw-py-1 tw-rounded tw-transition-colors"
          >
            Clear Log
          </button>
        </div>
      </div>
      
      <div class="tw-space-y-3">
        <app-author-detail
          *ngFor="let author of authors; trackBy: trackByAuthorId"
          [author]="author"
          (deleteAuthor)="handleDelete($event)"
        ></app-author-detail>
      </div>
      
      <div *ngIf="authors.length === 0" class="tw-text-center tw-py-12 tw-bg-gray-50 tw-rounded-lg tw-border-2 tw-border-dashed tw-border-gray-300">
        <div class="tw-text-6xl tw-mb-4">📝</div>
        <p class="tw-text-gray-500 tw-text-xl tw-mb-2">No authors available</p>
        <p class="tw-text-sm tw-text-gray-400 tw-mb-4">All authors have been deleted</p>
        <button 
          (click)="resetAuthors()"
          class="tw-bg-white hover:tw-bg-gray-100 tw-text-black tw-border-2 tw-border-black tw-px-6 tw-py-2 tw-rounded-lg tw-transition-colors tw-font-bold"
        >
          🔄 Restore Authors
        </button>
      </div>
    </div>
  `,
  styles: []
})
export class AuthorListComponent implements OnInit {
  authors: Author[] = [...authors]; // Tạo copy để có thể modify
  eventLog: string[] = [];

  constructor() {}

  ngOnInit(): void {
    this.logEvent('Author List component initialized');
  }

  handleDelete(author: Author): void {
    this.logEvent(`🗑️ Delete event received for: ${author.firstName} ${author.lastName} (ID: ${author.id})`);
    this.authors = this.authors.filter(item => item.id !== author.id);
    this.logEvent(`✅ Author deleted successfully. Remaining: ${this.authors.length} authors`);
  }

  resetAuthors(): void {
    this.authors = [...authors];
    this.logEvent('🔄 Authors list reset to initial state');
  }

  clearEventLog(): void {
    this.eventLog = [];
  }

  logEvent(message: string): void {
    const timestamp = new Date().toLocaleTimeString();
    this.eventLog.push(`[${timestamp}] ${message}`);
    console.log('Event:', message);
  }

  trackByAuthorId(index: number, author: Author): number {
    return author.id;
  }
}
