import { Component, OnInit, OnDestroy } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject, takeUntil } from 'rxjs';
import { CartService } from '../../../shared/data-access/service/cart.service';
import { CartItem } from '../../../shared/data-access/interface/product.interface';

/**
 * CartViewComponent - Demo Service Sharing
 * 
 * Component này share cùng CartService instance với ProductListComponent
 * Minh chứng tính singleton của service khi providedIn: 'root'
 */
@Component({
  selector: 'app-cart-view',
  standalone: true,
  imports: [CommonModule],
  templateUrl: './cart-view.component.html',
  styleUrls: ['./cart-view.component.scss']
})
export class CartViewComponent implements OnInit, OnDestroy {
  cartItems: CartItem[] = [];
  cartCount = 0;
  cartTotal = 0;
  cartSubtotal = 0;
  cartDiscount = 0;

  private destroy$ = new Subject<void>();

  /**
   * Constructor - Inject cùng CartService instance
   */
  constructor(private cartService: CartService) {
    console.log('🛒 CartViewComponent constructor called');
    console.log('🔗 CartService injected (same instance):', !!this.cartService);
  }

  ngOnInit(): void {
    console.log('🚀 CartViewComponent ngOnInit');
    this.subscribeToCartChanges();
  }

  ngOnDestroy(): void {
    this.destroy$.next();
    this.destroy$.complete();
  }

  /**
   * Subscribe to cart changes từ shared CartService
   */
  private subscribeToCartChanges(): void {
    // Subscribe to cart items
    this.cartService.cartItems$
      .pipe(takeUntil(this.destroy$))
      .subscribe((items: CartItem[]) => {
        this.cartItems = items;
        this.updateCartTotals();
        console.log('🔄 Cart items updated in CartView:', items.length);
      });

    // Subscribe to cart count
    this.cartService.cartCount$
      .pipe(takeUntil(this.destroy$))
      .subscribe((count: number) => {
        this.cartCount = count;
      });
  }

  /**
   * Update cart totals
   */
  private updateCartTotals(): void {
    this.cartSubtotal = this.cartItems.reduce(
      (total, item) => total + (item.product.price * item.quantity),
      0
    );
    this.cartTotal = this.cartService.calculateTotal();
    this.cartDiscount = this.cartSubtotal - this.cartTotal;
  }

  /**
   * Update quantity sử dụng shared CartService
   */
  updateQuantity(productId: number, quantity: number): void {
    console.log(`📝 Updating quantity for product ${productId}: ${quantity}`);
    this.cartService.updateQuantity(productId, quantity);
  }

  /**
   * Remove item sử dụng shared CartService
   */
  removeItem(productId: number): void {
    console.log(`🗑️ Removing product ${productId} from cart`);
    this.cartService.removeFromCart(productId);
  }

  /**
   * Clear cart sử dụng shared CartService
   */
  clearCart(): void {
    if (confirm('Bạn có chắc chắn muốn xóa toàn bộ giỏ hàng?')) {
      console.log('🧹 Clearing entire cart');
      this.cartService.clearCart();
    }
  }

  /**
   * Increment quantity
   */
  incrementQuantity(productId: number, currentQuantity: number): void {
    this.updateQuantity(productId, currentQuantity + 1);
  }

  /**
   * Decrement quantity
   */
  decrementQuantity(productId: number, currentQuantity: number): void {
    if (currentQuantity > 1) {
      this.updateQuantity(productId, currentQuantity - 1);
    } else {
      this.removeItem(productId);
    }
  }

  /**
   * Handle image error
   */
  onImageError(event: any): void {
    event.target.src = 'https://picsum.photos/80/80?random=888';
  }

  /**
   * TrackBy function for ngFor optimization
   */
  trackByProductId(index: number, item: CartItem): number {
    return item.product.id;
  }
}
