import { Component, OnInit, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject, takeUntil } from 'rxjs';
import { ProductService } from '../../../shared/data-access/service/product.service';
import { CartService } from '../../../shared/data-access/service/cart.service';
import { ProductModel } from '../../../shared/data-access/interface/product.interface';

/**
               * ProductListComponent - Product list with cart
 */
@Component({
  selector: 'app-product-list',
  standalone: true,
  imports: [CommonModule],
  templateUrl: './product-list.component.html',
  styleUrls: ['./product-list.component.scss']
})
export class ProductListComponent implements OnInit, OnDestroy {
  products: ProductModel[] = [];
  loading = true;
  cartCount = 0;
  cartTotal = 0;
  addingToCart: { [productId: number]: boolean } = {};

  private destroy$ = new Subject<void>();

  constructor(
    private productService: ProductService,
    private cartService: CartService,
    private cdr: ChangeDetectorRef
  ) {
    console.log('🎯 ProductListComponent initialized');
  }

  ngOnInit(): void {
    this.loadProducts();
    this.subscribeToCartChanges();
  }

  ngOnDestroy(): void {
    this.destroy$.next();
    this.destroy$.complete();
  }

  private loadProducts(): void {
    this.productService.getAllProducts()
      .pipe(takeUntil(this.destroy$))
      .subscribe({
        next: (products: ProductModel[]) => {
          this.products = products;
          this.loading = false;
          console.log('✅ Products loaded:', products.length);
        },
        error: (error: any) => {
          console.error('❌ Error loading products:', error);
          this.loading = false;
        }
      });
  }

  private subscribeToCartChanges(): void {
    this.cartService.cartCount$
      .pipe(takeUntil(this.destroy$))
      .subscribe((count: number) => {
        this.cartCount = count;
      });

    this.cartService.cartItems$
      .pipe(takeUntil(this.destroy$))
      .subscribe(() => {
        this.cartTotal = this.cartService.calculateTotal();
      });
  }

  addToCart(product: ProductModel): void {
    console.log('🛒 Adding product to cart:', product.name);
    console.log('🔍 Current loading state for product', product.id, ':', this.addingToCart[product.id]);
    
    // Prevent multiple clicks
    if (this.addingToCart[product.id]) {
      console.log('⚠️ Already adding to cart, skipping...');
      return;
    }
    
    // Set loading state
    this.addingToCart[product.id] = true;
    console.log('🔄 Set loading state to true for product', product.id);

    try {
      // Add to cart
      this.cartService.addToCart(product, 1);
      console.log('✅ Product added to cart successfully');
    } catch (error) {
      console.error('❌ Error adding to cart:', error);
    }
    
    // Reset loading state after a short delay for UX
    setTimeout(() => {
      console.log('🔄 Resetting loading state for product', product.id);
      this.addingToCart[product.id] = false;
      this.cdr.detectChanges(); // Force change detection
      console.log('✅ Loading state reset for product', product.id);
    }, 200);
  }

  onImageError(event: any): void {
    event.target.src = 'https://picsum.photos/200/200?random=999';
  }
}
